from django.contrib import admin

from tos.models import TermsOfService, UserAgreement


class TermsOfServiceAdmin(admin.ModelAdmin):
    model = TermsOfService

admin.site.register(TermsOfService, TermsOfServiceAdmin)


class UserAgreementAdmin(admin.ModelAdmin):
    model = UserAgreement

admin.site.register(UserAgreement, UserAgreementAdmin)



from django.apps import AppConfig
from django.conf import settings
from django.core.cache import caches
from django.db.models.signals import pre_save
from django.dispatch import receiver


MIDDLEWARES = getattr(settings, 'MIDDLEWARE_CLASSES', [])


class TOSConfig(AppConfig):
    name = 'tos'
    verbose_name = 'Terms Of Service'

    def ready(self):
        if 'tos.middleware.UserAgreementMiddleware' in MIDDLEWARES:
            # Force the user to create a separate cache
            cache = caches[getattr(settings, 'TOS_CACHE_NAME', 'default')]

            TermsOfService = self.get_model('TermsOfService')

            @receiver(pre_save, sender=TermsOfService, dispatch_uid='invalidate_cached_agreements')
            def invalidate_cached_agreements(TermsOfService, instance, **kwargs):
                if kwargs.get('raw', False):
                    return

                # Set the key version to 0 if it doesn't exist and leave it
                # alone if it does
                cache.add('django:tos:key_version', 0)

                # This key will be used to version the rest of the TOS keys
                # Incrementing it will effectively invalidate all previous keys
                cache.incr('django:tos:key_version')

            # Create the TOS key version immediately
            invalidate_cached_agreements(TermsOfService, None)



from django.conf import settings
from django.core.urlresolvers import reverse
from django.test import TestCase

from tos.compat import get_runtime_user_model
from tos.models import TermsOfService, UserAgreement, has_user_agreed_latest_tos


class TestViews(TestCase):

    def setUp(self):
        self.user1 = get_runtime_user_model().objects.create_user('user1', 'user1@example.com', 'user1pass')
        self.user2 = get_runtime_user_model().objects.create_user('user2', 'user2@example.com', 'user2pass')

        self.tos1 = TermsOfService.objects.create(
            content="first edition of the terms of service",
            active=True
        )
        self.tos2 = TermsOfService.objects.create(
            content="second edition of the terms of service",
            active=False
        )

        self.login_url = getattr(settings, 'LOGIN_URL', '/login/')

        UserAgreement.objects.create(
            terms_of_service=self.tos1,
            user=self.user1
        )

    def test_login(self):
        """ Make sure we didn't break the authentication system
            This assumes that login urls are named 'login'
        """

        self.assertTrue(has_user_agreed_latest_tos(self.user1))
        login = self.client.login(username='user1', password='user1pass')
        self.failUnless(login, 'Could not log in')
        self.assertTrue(has_user_agreed_latest_tos(self.user1))

    def test_need_agreement(self):
        """ user2 tries to login and then has to go and agree to terms"""

        self.assertFalse(has_user_agreed_latest_tos(self.user2))

        response = self.client.post(self.login_url, dict(username='user2', password='user2pass'))
        self.assertContains(response, "first edition of the terms of service")

        self.assertFalse(has_user_agreed_latest_tos(self.user2))

    def test_do_not_need_agreement(self):
        """ user2 tries to login and has already agreed"""

        self.assertTrue(has_user_agreed_latest_tos(self.user1))

        response = self.client.post(self.login_url, dict(username='user1',
            password='user1pass'))
        self.assertEqual(302, response.status_code)

    def test_redirect_security(self):
        """ redirect to outside url not allowed, should redirect to login url"""

        response = self.client.post(self.login_url, dict(username='user1',
            password='user1pass', next='http://example.com'))
        self.assertEqual(302, response.status_code)
        self.assertIn(settings.LOGIN_REDIRECT_URL, response._headers['location'][1])

    def test_need_to_log_in(self):
        """ GET to login url shows login tempalte."""

        response = self.client.get(self.login_url)
        self.assertContains(response, "Dummy login template.")

    def test_root_tos_view(self):

        response = self.client.get('/tos/')
        self.assertIn(b'first edition of the terms of service', response.content)

    def test_reject_agreement(self):

        self.assertFalse(has_user_agreed_latest_tos(self.user2))

        response = self.client.post(self.login_url, dict(username='user2', password='user2pass'))
        self.assertContains(response, "first edition of the terms of service")
        url = reverse('tos_check_tos')
        response = self.client.post(url, {'accept': 'reject'})

        self.assertFalse(has_user_agreed_latest_tos(self.user2))

    def test_accept_agreement(self):

        self.assertFalse(has_user_agreed_latest_tos(self.user2))

        response = self.client.post(self.login_url, dict(username='user2', password='user2pass'))
        self.assertContains(response, "first edition of the terms of service")
        self.assertFalse(has_user_agreed_latest_tos(self.user2))
        url = reverse('tos_check_tos')
        response = self.client.post(url, {'accept': 'accept'})

        self.assertTrue(has_user_agreed_latest_tos(self.user2))

    def test_bump_new_agreement(self):

        # Change the tos
        self.tos2.active = True
        self.tos2.save()

        # is user1 agreed now?
        self.assertFalse(has_user_agreed_latest_tos(self.user1))

        # user1 agrees again
        response = self.client.post(self.login_url, dict(username='user1', password='user1pass'))
        self.assertContains(response, "second edition of the terms of service")
        self.assertFalse(has_user_agreed_latest_tos(self.user2))
        url = reverse('tos_check_tos')
        response = self.client.post(url, {'accept': 'accept'})

        self.assertTrue(has_user_agreed_latest_tos(self.user1))



from django.conf.urls import url

from tos.compat import patterns
from tos.views import check_tos, TosView


urlpatterns = patterns('',
    # Terms of Service conform
    url(r'^confirm/$', check_tos, name='tos_check_tos'),

    # Terms of service simple display
    url(r'^$', TosView.as_view(), name='tos'),
)



from django.contrib import admin
from modeltranslation.admin import TranslationAdmin
from tos.admin import TermsOfServiceAdmin

# Admin translation for django-plans
from tos.models import TermsOfService


class TranslatedTermsOfServiceAdmin(TermsOfServiceAdmin, TranslationAdmin):
    pass

admin.site.unregister(TermsOfService)
admin.site.register(TermsOfService, TranslatedTermsOfServiceAdmin)



from django.core.exceptions import ValidationError
from django.test import TestCase

from tos.compat import get_runtime_user_model
from tos.models import (
                        NoActiveTermsOfService,
                        TermsOfService,
                        UserAgreement,
                        has_user_agreed_latest_tos,
                       )


class TestModels(TestCase):

    def setUp(self):
        self.user1 = get_runtime_user_model().objects.create_user('user1',
                                                    'user1@example.com',
                                                    'user1pass')
        self.user2 = get_runtime_user_model().objects.create_user('user2',
                                                    'user2@example.com',
                                                    'user2pass')
        self.user3 = get_runtime_user_model().objects.create_user('user3',
                                                    'user3@example.com',
                                                    'user3pass')

        self.tos1 = TermsOfService.objects.create(
            content="first edition of the terms of service",
            active=True
        )
        self.tos2 = TermsOfService.objects.create(
            content="second edition of the terms of service",
            active=False
        )

    def test_terms_of_service(self):

        self.assertEquals(TermsOfService.objects.count(), 2)

        # order is by -created
        latest = TermsOfService.objects.latest()
        self.assertFalse(latest.active)

        # setting a tos to True changes all others to False
        latest.active = True
        latest.save()
        first = TermsOfService.objects.get(id=self.tos1.id)
        self.assertFalse(first.active)

        # latest is active though
        self.assertTrue(latest.active)

    def test_validation_error_all_set_false(self):
        """
        If you try and set all to false the model will throw a ValidationError
        """

        self.tos1.active = False
        self.assertRaises(ValidationError, self.tos1.save)

    def test_user_agreement(self):

        # simple agreement
        UserAgreement.objects.create(
            terms_of_service=self.tos1,
            user=self.user1
        )

        self.assertTrue(has_user_agreed_latest_tos(self.user1))
        self.assertFalse(has_user_agreed_latest_tos(self.user2))
        self.assertFalse(has_user_agreed_latest_tos(self.user3))

        # Now set self.tos2.active to True and see what happens
        self.tos2.active = True
        self.tos2.save()
        self.assertFalse(has_user_agreed_latest_tos(self.user1))
        self.assertFalse(has_user_agreed_latest_tos(self.user2))
        self.assertFalse(has_user_agreed_latest_tos(self.user3))

        # add in a couple agreements and try again
        UserAgreement.objects.create(
            terms_of_service=self.tos2,
            user=self.user1
        )
        UserAgreement.objects.create(
            terms_of_service=self.tos2,
            user=self.user3
        )

        self.assertTrue(has_user_agreed_latest_tos(self.user1))
        self.assertFalse(has_user_agreed_latest_tos(self.user2))
        self.assertTrue(has_user_agreed_latest_tos(self.user3))


class TestManager(TestCase):
    def test_terms_of_service_manager(self):

        tos1 = TermsOfService.objects.create(
            content="first edition of the terms of service",
            active=True
        )

        self.assertEquals(TermsOfService.objects.get_current_tos(), tos1)

    def test_terms_of_service_manager_raises_error(self):

        self.assertRaises(NoActiveTermsOfService, TermsOfService.objects.get_current_tos)



import django
from django.conf import settings


def patterns(mod, *urls):
    if mod != '' or django.VERSION < (1, 9):
        from django.conf.urls import patterns
        return patterns(mod, *urls)
    else:
        return list(urls)


def get_fk_user_model():
    if django.VERSION >= (1, 5):
        return settings.AUTH_USER_MODEL
    from django.contrib.auth.models import User
    return User


def get_runtime_user_model():
    if django.VERSION >= (1, 5):
        from django.contrib.auth import get_user_model
        return get_user_model()
    from django.contrib.auth.models import User
    return User


def get_request_site():
    if django.VERSION >= (1, 9):
        from django.contrib.sites.requests import RequestSite
    else:
        from django.contrib.sites.models import RequestSite
    return RequestSite


def get_library():
    if django.VERSION >= (1, 9):
        from django.template.library import Library
    else:
        from django.template.base import Library
    return Library


if django.VERSION < (1, 5):
    from django.templatetags.future import url
else:
    from django.template.defaulttags import url



from django.views.generic import TemplateView
import re
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import login as auth_login
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.sites.models import Site
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect
from django.utils.translation import ugettext_lazy as _

from tos.compat import get_runtime_user_model, get_request_site
from tos.models import has_user_agreed_latest_tos, TermsOfService, UserAgreement


class TosView(TemplateView):
    template_name = "tos/tos.html"

    def get_context_data(self, **kwargs):
        context = super(TosView, self).get_context_data(**kwargs)
        context['tos'] = TermsOfService.objects.get_current_tos()
        return context


def _redirect_to(redirect_to):
    """ Moved redirect_to logic here to avoid duplication in views"""

    # Light security check -- make sure redirect_to isn't garbage.
    if not redirect_to or ' ' in redirect_to:
        redirect_to = settings.LOGIN_REDIRECT_URL

    # Heavier security check -- redirects to http://example.com should
    # not be allowed, but things like /view/?param=http://example.com
    # should be allowed. This regex checks if there is a '//' *before* a
    # question mark.
    elif '//' in redirect_to and re.match(r'[^\?]*//', redirect_to):
            redirect_to = settings.LOGIN_REDIRECT_URL
    return redirect_to


@csrf_protect
@never_cache
def check_tos(request, template_name='tos/tos_check.html',
              redirect_field_name=REDIRECT_FIELD_NAME,):

    redirect_to = _redirect_to(request.POST.get(redirect_field_name, request.GET.get(redirect_field_name, '')))
    tos = TermsOfService.objects.get_current_tos()
    if request.method == "POST":
        if request.POST.get("accept", "") == "accept":
            user = get_runtime_user_model().objects.get(pk=request.session['tos_user'])
            user.backend = request.session['tos_backend']

            # Save the user agreement to the new TOS
            UserAgreement.objects.create(terms_of_service=tos, user=user)

            # Log the user in
            auth_login(request, user)

            if request.session.test_cookie_worked():
                request.session.delete_test_cookie()

            return HttpResponseRedirect(redirect_to)
        else:
            messages.error(
                request,
                _(u"You cannot login without agreeing to the terms of this site.")
            )

    return render_to_response(template_name, {
        'tos': tos,
        redirect_field_name: redirect_to,
    }, context_instance=RequestContext(request))


@csrf_protect
@never_cache
def login(request, template_name='registration/login.html',
          redirect_field_name=REDIRECT_FIELD_NAME,
          authentication_form=AuthenticationForm):
    """Displays the login form and handles the login action."""

    redirect_to = request.POST.get(redirect_field_name, request.GET.get(redirect_field_name, ''))

    if request.method == "POST":
        form = authentication_form(data=request.POST)
        if form.is_valid():

            redirect_to = _redirect_to(redirect_to)

            # Okay, security checks complete. Check to see if user agrees
            # to terms
            user = form.get_user()
            if has_user_agreed_latest_tos(user):

                # Log the user in.
                auth_login(request, user)

                if request.session.test_cookie_worked():
                    request.session.delete_test_cookie()

                return HttpResponseRedirect(redirect_to)

            else:
                # user has not yet agreed to latest tos
                # force them to accept or refuse

                request.session['tos_user'] = user.pk
                # Pass the used backend as well since django will require it
                # and it can only be optained by calling authenticate, but we
                # got no credentials in check_tos.
                # see: https://docs.djangoproject.com/en/1.6/topics/auth/default/#how-to-log-a-user-in
                request.session['tos_backend'] = user.backend

                return render_to_response('tos/tos_check.html', {
                    redirect_field_name: redirect_to,
                    'tos': TermsOfService.objects.get_current_tos()
                }, context_instance=RequestContext(request))

    else:
        form = authentication_form(request)

    request.session.set_test_cookie()

    if Site._meta.installed:
        current_site = Site.objects.get_current()
    else:
        current_site = get_request_site()(request)

    return render_to_response(template_name, {
        'form': form,
        redirect_field_name: redirect_to,
        'site': current_site,
        'site_name': current_site.name,
    }, context_instance=RequestContext(request))



from django.conf import settings
from django.contrib.auth import SESSION_KEY as session_key
from django.core.cache import caches
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.cache import add_never_cache_headers

from .models import UserAgreement

cache = caches[getattr(settings, 'TOS_CACHE_NAME', 'default')]
tos_check_url = reverse('tos_check_tos')


class UserAgreementMiddleware(object):
    """
    Some middleware to check if users have agreed to the latest TOS
    """
    def process_request(self, request):
        # Don't get in the way of any mutating requests
        if request.method != 'GET':
            return None

        # Ignore ajax requests
        if request.is_ajax():
            return None

        # Don't redirect users when they're trying to get to the confirm page
        if request.path_info == tos_check_url:
            return None

        # If the user doesn't have a user ID, ignore them - they're anonymous
        if not request.session.get(session_key, None):
            return None

        # Grab the user ID from the session so we avoid hitting the database
        # for the user object.
        # NOTE: We use the user ID because it's not user-settable and it won't
        #       ever change (usernames and email addresses can change)
        user_id = request.session.get(session_key)

        # Get the cache prefix
        key_version = cache.get('django:tos:key_version')

        # Skip if the user is allowed to skip - for instance, if the user is an
        # admin or a staff member
        if cache.get('django:tos:skip_tos_check:{}'.format(str(user_id)), False, version=key_version):
            return None

        # Ping the cache for the user agreement
        user_agreed = cache.get('django:tos:agreed:{}'.format(str(user_id)), None, version=key_version)

        # If the cache is missing this user
        if user_agreed is None:
            # Grab the data from the database
            user_agreed = UserAgreement.objects.filter(
                user__id=user_id,
                terms_of_service__active=True).exists()

            # Set the value in the cache
            cache.set('django:tos:agreed:{}'.format(user_id), user_agreed, version=key_version)

        if not user_agreed:
            return add_never_cache_headers(HttpResponseRedirect(tos_check_url))

        return None
